/**
 * Function.js
 * version 1.2
 *
 * contributors:
 *  Jason Staples (most of it)
 *  Shoel Perelman
 *  Adrian Fowler (log function from Webtop's ActionUtils)
 */

var separator="/";
var extension = "";
var i=0;
var arch = "";
var ncp = ncipackage;
if (ncp != null) {
	arch = ncipackage.shortArch;
}

if (arch == "win32") {
	separator="\\";
  	extension = ".bat";
}

function initialize() {
	var gPackageManager = Components.classes["@micromuse.com/NCIPackageManager;1"].getService(Components.interfaces.nsINCIPackageManager);
	arch = gPackageManager.products.item(0).packages.item(0).shortArch;

	if (arch == "win32") {
  	separator="\\";
  	extension = ".bat";
	}
}


/**
 * Log a message. -- From webtop's ActionUtils.js
 *
 * @param level (int) The level to log with. Valid levels are: 0 - Error,
 *		  1 - Warning 2 - Info, 3 - Debug.
 * @param msg (String) The message to log.
 */
function log( level, msg )
{
	try
	{
		var logger = Components.classes["@micromuse.com/NCILog;1"].
			getService( Components.interfaces.nsINCILog );
		switch ( level )
		{
			case 0: logger.error( msg ); break;
			case 1: logger.warning( msg ); break;
			case 2: logger.info( msg ); break;
			case 3: logger.debug( msg ); break;
			default: break;
		}
	}
	catch ( e )
	{
		// Logging failure must not affect functionality, so fail silently.
	}
}

function getHostName() {
	var DNSService = Components.classes["@mozilla.org/network/dns-service;1"].getService(Components.interfaces.nsIDNSService);
     	var hostName = DNSService.myHostName;
	return hostName;
}

function appendFile(sourcefile, appendstring) {
  log(3, "appendFile(sourcefile: "+sourcefile+", appendstring: "+appendstring+")");

  // file is nsIFile, data is a string
  var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile );
  file.initWithPath(sourcefile);

  if (file.exists()) {
    log(3, "appendFile setting permissions for existing file to | 0x7 for sourcefile: "+sourcefile);
    file.permissions = file.permissions | 0777;
  }

  log(3, "appendFile opened sourcefile: "+sourcefile);
  var foStream = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(Components.interfaces.nsIFileOutputStream);
  //log(0, "got output stream");
  // use 0x02 | 0x10 to open file for appending.
  foStream.init(file, 0x04 | 0x08 | 0x10, 0777, 0); // write, create, truncate
  log(3, "appendFile initialized foStream for sourcefile: "+sourcefile);

  log(3, "appendFile setting new permissions to | 0x7 for sourcefile: "+sourcefile);
  file.permissions = file.permissions | 0777;



  //log(0, "file init");
  foStream.write(appendstring, appendstring.length);
  log(3, "appendFile wrote data: "+appendstring);
  foStream.close();
  log(3, "appendFile closed file");
  return true;
}

function makeDir(directory) {
  var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile );
  file.initWithPath(directory);
  if (!file.exists()) {
    file.create(1, 0777);
  }
}

function readFile(filename) {
  var data = "";
  var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile );
  file.initWithPath(filename);
  if (file.exists()) {
    var fstream = Components.classes["@mozilla.org/network/file-input-stream;1"].createInstance(Components.interfaces.nsIFileInputStream);
    var sstream = Components.classes["@mozilla.org/scriptableinputstream;1"].createInstance(Components.interfaces.nsIScriptableInputStream);
    fstream.init(file, 1, 0, false);
    sstream.init(fstream); 
    data += sstream.read(-1);
    sstream.close();
    fstream.close();
  }
  return data;
}

function deleteFile(filename, recursive) {
  var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile );
  file.initWithPath(filename);
  if (file.exists()) {
    file.remove(recursive);
  }
}

function copyFile(srcfileName, destdirName, destfileName) {
  var srcfile = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile );
  try {
    srcfile.initWithPath(srcfileName);
  } catch (e) {
    log(0, "copyFile("+srcFileName+", "+destdirName+", " +destfileName+") Can't find source file: " + srcfileName+": "+e);
    return false;
  }
  
  var outfile = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile );

  try {
    outfile.initWithPath(destdirName + separator + destfileName);
    outfile.remove(true);
  } catch (e) {
    log(3, "Can't find file to remove" + destdirName + separator + destfileName);
  }

  var destdir = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile );

  try {
    destdir.initWithPath(destdirName);
  } catch (e) {
    log(0, "copyFile("+srcFileName+", "+destdirName+", " +destfileName+") Can't find destination dir: " + destdirName+": "+e);
    return false;
  }

  try {
    srcfile.copyTo(destdir, destfileName);
  } catch (e) {
    log(0, "Can't copy file " + destdir + separator + destfileName + " Error: " + e);
    return false;
  }

  return true;
}



/**
 * Copy all files from srcDirName to destdirName.  Note that destDirName
 * must already exist (create it with makeDir() or createDirWithBackup())
 */
function copyDirContents(srcDirName, destDirName) {
  log(3, "copyDirContents( srcDirName: "+srcDirName+", destDirName: "+destDirName+")...");
  var srcdir = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile );
  try {
    srcdir.initWithPath(srcDirName);
  } catch (e) {
    log(0, "copyDirContents("+srcDirName+", "+destDirName+") Can't find source dir: " + srcDirName+": "+e);
    return false;
  }

  var destdir = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile );

  try {
    destdir.initWithPath(destDirName);
  } catch (e) {
    log(0, "copyDirContents("+srcDirName+", "+destDirName+") Can't find dest dir: " + destDirName+": "+e);
    return false;
  }

  // get directory entries
  var directoryEntries = srcdir.directoryEntries;
  log(3, "copyDirContents directory entries: "+directoryEntries);
  while(directoryEntries.hasMoreElements()) {
    var entry = directoryEntries.getNext();
    var file = entry.QueryInterface(Components.interfaces.nsIFile);
    log(3, "copyDirContents copying: "+file.leafName);
    file.copyTo(destdir, null);
  }
}




function searchAndReplace(filename, searchstring) {
  log(3, "searchAndReplace(file: "+filename+", searchstring: "+searchstring);

  try {
    var infile = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile );
    infile.initWithPath(filename);
    //log(0, "created infile");
  } catch (e) {
    log(0, "Can't init with file " + filename+": "+e);
    return false;
  }

  try {
    //log(0, "creating infileStream");
    var infileStream = Components.classes["@mozilla.org/network/file-input-stream;1"].createInstance(Components.interfaces.nsIFileInputStream);
    //log(0, "created infileStream");
    infileStream.init(infile, 0x01, 0755, 0);
    //log(0, "init infileStream");
    infileStream.QueryInterface(Components.interfaces.nsILineInputStream);
  } catch (e) {
    log(0, "Can't create file stream for file " + filename+":" +e);
    return false;
  }

  try {
    var outfile = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile );
    outfile.initWithPath(filename + ".new");
  } catch (e) {
    log(0, "Can't init output file " + filename+": " +e);
    return false;
  }

  try {
    var outfileStream = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(Components.interfaces.nsIFileOutputStream);
    outfileStream.init(outfile, 0x02 | 0x08 | 0x10, 0755, 0);
  } catch (e) {
    log(0, "Can't create output file stream for file " + filename+": "+e);
    return false;
  }

  var line = {};
  var hasmore;

  do {
    hasmore = infileStream.readLine(line);
    currentline = line.value;
    currentline = searchAndReplaceString(currentline, searchstring);

    currentline = currentline + '\n';
    //log(0, "after replace: " + currentline);
    outfileStream.write(currentline, currentline.length);   
  } while (hasmore);
  
  outfileStream.close();
  infileStream.close();
  
  //log(0, infile.parent.path);
  //log(0, infile.leafName);
  
  //log(0, "infile permission" + infile.permissions);
  infile.remove(true);
  outfile.copyTo(null, infile.leafName);
  //log(0, "outfile permission" + outfile.permissions);
  outfile.remove(true);
  return true;
}

function searchAndReplaceString(instring, searchstring) {
  var sPairs = searchstring.split("%%");
  if (instring != "" ) {
    for (var i=0; i<sPairs.length; i++) {
      if (sPairs[i] != "") {
	sNV = sPairs[i].split("^");
	if (sNV.length = 2) {
	  var regex = new RegExp(sNV[0], "g");
	  instring = instring.replace(regex, sNV[1]);    
	}
      }
    }
  }
  return instring;
}

function execScript(script) {
  log(0, "running execScript");

  var propMgr = Components.classes["@micromuse.com/NCIProperties;1"].getService(Components.interfaces.nsINCIProperties);

  prop = propMgr.GetProp("$NCHOME");
  var nchome = prop.value;
  
  makeDir(nchome + separator + "tmp");

  var scriptname = nchome + separator + "tmp" + separator + "execpro" + i + extension;
  // create temp file
  appendFile(scriptname, script + "\n");
  
  var outputscript = nchome + separator + "tmp" + separator + "runexecpro" + i + extension;
  if (arch == "win32") {
    appendFile(outputscript, "@echo off\n");
  } else {
    appendFile(outputscript, "#!/bin/sh\n");
  }
  appendFile(outputscript, "\"" + scriptname + "\" >> \"" + outputscript + ".out\" 2>\"" + outputscript + ".err\"\n");

  // create an nsILocalFile for the executable
  var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
  file.initWithPath(outputscript);
  //file.initWithPath("c:\\windows\\system32\\cmd.exe");

  // create an nsIProcess
  var process = Components.classes["@mozilla.org/process/util;1"].createInstance(Components.interfaces.nsIProcess);
  process.init(file);

  // Run the process.
  // If first param is true, calling process will be blocked until
  // called process terminates. 
  // Second and third params are used to pass command-line arguments
  // to the process.
  var args = [];
  var outvalue = process.run(true, args, args.length);

  var output = new Object();
  output.stdout = readFile(outputscript + ".out");
  output.stderr = readFile(outputscript + ".err");
  deleteFile(scriptname, true);
  deleteFile(outputscript, true);
  deleteFile(outputscript + ".out", true);
  deleteFile(outputscript + ".err", true);
  i++;
  return output;
}

function runCrypt(password) {
  var script = "";
  var log = Components.classes["@micromuse.com/NCILog;1"].getService(Components.interfaces.nsINCILog);
  var propMgr = Components.classes["@micromuse.com/NCIProperties;1"].getService(Components.interfaces.nsINCIProperties);

  prop = propMgr.GetProp("NCHOME.INSTALLDIR");
  var nchome = prop.value;
  var ncsmhome = prop.value + separator + "security";
  
  var cmd = "\"" + ncsmhome + separator + "bin" + separator + "ncsm_crypt" + extension + "\" \"" + password + "\"";
  //log(0, "runcrypt password = " + password);
  if (arch == "win32") {
    script = "set NCHOME=" + nchome + "\n" + cmd;
  } else {
    script = "NCSM_HOME=" + ncsmhome + "\nexport NCSM_HOME\n" + cmd;
  }

  var output = execScript(script);
  //log(0, "runcrypt stdout = " + output.stdout);
  return output.stdout
}


// Functions added by SDP 13-Jun-05

// Returns true if port is in use, false if it is not
function isPortInUse(host, port) {
  netscape.security.PrivilegeManager.enablePrivilege("UniversalBrowserRead UniversalBrowserWrite UniversalXPConnect");
  var nsXMLHTTP = Components.classes["@mozilla.org/xmlextras/xmlhttprequest;1"].createInstance(Components.interfaces.nsIXMLHttpRequest);

  // make a fake url and try to connect to it
  var url = "http://"+host+":"+port;

  var portInUse = false;

  try { 
    nsXMLHTTP.open("GET", url, false);
    nsXMLHTTP.send(null);
    portInUse = true; // if we got here, it worked
  } catch (e) {
    // error opening connection
    portInUse = false;
  }

  return portInUse;
}


/**
 * Returns a directory without a trailing slash (if it ends with one).
 */
function getPathWithoutTrailingSlash(fullPath) {
  log(3, "getPathWithoutTrailingSlash("+fullPath+")");
  if (fullPath.charAt(fullPath.length-1) == separator) {
    return fullPath.substring(0, (fullPath.length-1));
  }
  return fullPath;
}


/**
 * Returns the base name of a full path (section after last slash)
 */
function getBaseName(fullPath) {
  log(3, "getBaseName("+fullPath+")");
  var pathWithoutLastSlash = getPathWithoutTrailingSlash(fullPath);
  var lastSlashPos = pathWithoutLastSlash.lastIndexOf(separator) + 1;
  log(3, "getBaseName("+fullPath+") returning "+ fullPath.substring(lastSlashPos,fullPath.length));
  return fullPath.substring(lastSlashPos,fullPath.length);
}


/**
 * Creates an empty directory.  Will rename the dirName to dirName.bak
 * if it already exists. If dirName.bak exists, it'll rename it to 
 * dirName.1, .2, and so on.  Under no cicumstances will data be lost.
 */
function backupDirectory(fullDirName) {
  var dirName = getPathWithoutTrailingSlash(fullDirName);

  var file = Components.classes["@mozilla.org/file/local;1"].
    createInstance( Components.interfaces.nsILocalFile );
  
  
  log(3, "backupDirectory("+dirName+") checking if it already exists...");
  // first, check if dirname exists
  file.initWithPath( dirName );

  if (file.exists()) {
    log(3, "backupDirectory("+dirName+") -- it exists, must rename to .bak");
    // try to rename to .bak
    var appendNum = 0;

    // find a name to rename the current dir to
    var goodName = false;
    var appendBakStr = ".bak";
    while (!goodName) {
      appendBakStr = ".bak";
      if (appendNum > 0) {
	appendBakStr = appendBakStr + "." + appendNum;
      }
      
      file.initWithPath(dirName + appendBakStr);
      // does it still exist?
      
      log(3, "backupDirectory("+dirName+") checking if "+dirName + appendBakStr+" exists");
      if (file.exists()) {
	appendNum = appendNum + 1;
	log(3, "backupDirectory("+dirName+") " + dirName + appendBakStr+" exists also... Trying next.");
      } else {
	goodName = true;
	log(3, "backupDirectory("+dirName+") " + dirName + appendBakStr+" doesn't exist -- will rename to that");
      }
    }

    log(3, "backupDirectory("+dirName+") renaming existing dir to " + dirName + appendBakStr);
    // rename it to the good name we found
    file.initWithPath(dirName);
    file.moveTo(file.parent, getBaseName(dirName) + appendBakStr);
  }
}

function createDirWithBackup(fullDirName) {
  backupDirectory(fullDirName);
  var dirName = getPathWithoutTrailingSlash(fullDirName);

  var file = Components.classes["@mozilla.org/file/local;1"].createInstance( Components.interfaces.nsILocalFile );
  log(3, "createDirWithBackup("+dirName+") creating "+dirName);
  // now create the dir
  file.initWithPath(dirName);
  file.create( file.DIRECTORY_TYPE, 0775 );
  log(3, "createDirWithBackup("+dirName+") created "+dirName);
}

function createNTUser(username, password) {
  log(0, "running createNTUser");
  var cmd = "net user " + username + " " + password + " /add";
  script = cmd;
  var output = execScript(script);
  log(0, "stdout = " + output.stdout);
  log(0, "stderr = " + output.stderr);
  return output.stderr;
}

function verifyNTPassword(username, password) {
  var slashIndex = username.indexOf("\\");
  if (slashIndex == -1) {
	username = "%computername%\\" + username;
  }
  var cmd = "net use \\\\%computername%\\ipc$ /u:" + username + " " + password;
  cmd = cmd + "\nnet use \\\\%computername%\\ipc$ /d 2>NUL";
  script = cmd;
  var output = execScript(script);
  return output.stderr;
}
 
function verifyNTUser(username) {
  var domain = "";
  var slashIndex = username.indexOf("\\");
  if (slashIndex != -1 ) {
	username = username.substr(slashIndex+1);
	domain = " /domain";
  }
  var cmd = "net user " + username + domain;
  script=cmd;
  var output = execScript(script);
  return output.stderr;
}

function removeNTUser(username) {
  var cmd = "net user " + username + " /delete";
  script = cmd;
  var output = execScript(script);
  return output.stderr;
}
